"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MassFormatter = void 0;
const vscode = require("vscode");
class MassFormatter {
    constructor() {
        this.shouldCancel = false;
    }
    async formatAllFiles() {
        let u = 0;
        this.shouldCancel = false;
        let visited = [];
        while (true) {
            let editor = vscode.window.activeTextEditor;
            if (editor) {
                let filename = editor.document.fileName;
                if (filename.endsWith("json")) {
                    await vscode.commands.executeCommand("roomletool.overrideFormatWhileAllTabs");
                }
                if (this.shouldCancel || visited.indexOf(filename) > -1) {
                    break;
                }
                else {
                    visited.push(filename);
                }
                u++;
            }
            else {
                break;
            }
            await vscode.commands.executeCommand("workbench.action.nextEditor");
        }
        vscode.window.showInformationMessage("Formatted " + u + " files.");
        if (this.outputChannel !== undefined) {
            this.outputChannel.appendLine("Formatted " + u + " files.");
        }
    }
    async formatAllFilesInWorkspace(normalize) {
        let u = 0;
        this.shouldCancel = false;
        const files = await vscode.workspace.findFiles('**/*.json');
        const filesLength = files.length;
        let lastCountDone = 0;
        files.sort();
        if (this.massFormatCancelButton) {
            this.massFormatCancelButton.text = "MASS FORMATTING 0 / " + filesLength + " STOP";
        }
        for (var i = 0; i < filesLength; i++) {
            await vscode.workspace.openTextDocument(files[i]).then(async (doc) => {
                await vscode.window.showTextDocument(doc);
            });
            if (normalize) {
                await vscode.commands.executeCommand("roomletool.overrideFormatNormalizeWhileAllTabs");
            }
            else {
                await vscode.commands.executeCommand("roomletool.overrideFormatWhileAllTabs");
            }
            const activeEditor = vscode.window.activeTextEditor;
            if (activeEditor)
                await activeEditor.document.save();
            await vscode.commands.executeCommand('workbench.action.closeActiveEditor');
            u++;
            if (u >= lastCountDone + 10) {
                if (this.massFormatCancelButton) {
                    this.massFormatCancelButton.text = "MASS FORMATTING " + u + " / " + filesLength + " STOP";
                    lastCountDone = u;
                }
            }
            if (this.shouldCancel) {
                break;
            }
        }
        if (this.outputChannel !== undefined) {
            this.outputChannel.appendLine("Formatted " + u + " files.");
        }
        vscode.window.showInformationMessage("Format all JSONs in workspace finished.");
    }
}
exports.MassFormatter = MassFormatter;
//# sourceMappingURL=massformatter.js.map