"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RcjsonOutliner = void 0;
const vscode = require("vscode");
class RcjsonOutliner {
    constructor() {
        this.componentOrder = [
            "id", "comment", "label", "labels", "descriptions", "parameterGroups", "parameters", "subComponents",
            "onUpdate", "previewGeometry", "geometry", "geometryHD", "debugGeometry", "siblings", "parentDockings", "childDockings", "addOnSpots",
            "possibleChildren", "packageSizes", "packaging", "dimensionings", "valid", "articleNr", "pricing", "plugin-data"
        ];
    }
    async provideDocumentSymbols(document) {
        return new Promise((resolve) => {
            var symbols = [];
            let foundComponentAttributes = {};
            for (var i = 0; i < document.lineCount; i++) {
                var line = document.lineAt(i);
                var lineText = line.text;
                if (lineText.startsWith("    \"")) {
                    var split = lineText.split('\"');
                    if (split.length > 2) {
                        var foundAttribute = split[1];
                        foundComponentAttributes[foundAttribute] =
                            {
                                "lineAt": i,
                                "lineRange": line.range,
                                "lineText": lineText
                            };
                    }
                }
            }
            for (let i = 0; i < this.componentOrder.length; i++) {
                let sortedAttribute = this.componentOrder[i];
                if (foundComponentAttributes[sortedAttribute]) {
                    let symbol = new vscode.DocumentSymbol(i + ":\t" + sortedAttribute, '', vscode.SymbolKind.Function, foundComponentAttributes[sortedAttribute].lineRange, foundComponentAttributes[sortedAttribute].lineRange);
                    symbols.push(symbol);
                }
            }
            resolve(symbols);
        });
    }
}
exports.RcjsonOutliner = RcjsonOutliner;
//# sourceMappingURL=outliner.js.map