"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Normalizer = void 0;
class Normalizer {
    constructor() {
        this.componentOrder = [
            "id", "comment", "label", "labels", "descriptions", "parameterGroups", "parameters", "subComponents",
            "onUpdate", "previewGeometry", "geometry", "geometryHD", "debugGeometry", "siblings", "parentDockings", "childDockings", "addOnSpots",
            "possibleChildren", "packageSizes", "packaging", "dimensionings", "valid", "articleNr", "pricing", "plugin-data"
        ];
        this.labelsOrder = ["en", "de"];
        this.addOnSpotsOrder = ["position", "mask", "visible", "condition"];
        this.parameterGroupsOrder = ["key", "labels", "collapsed", "sort"];
        this.parametersOrder = [
            "key", "type", "global", "defaultValue", "labels", "group", "sort", "unitType",
            "enabled", "visible", "visibleAsGlobal", "visibleInPartList", "highlighted",
            "validGroups", "validRange", "validValues", "valueObjects", "conditionalGroups", "onValueChange"
        ];
        this.dockingsOrder = ["points", "ranges", "lines", "lineRanges"];
        this.connectionOrder = ["mask", "position", "rotation", "positionTo", "stepEnd", "stepX", "stepY", "stepZ", "condition", "onUpdate", "maxConnections", "maxChildren", "assignmentsOnDock", "assignmentsOnUpdate", "assignmentsOnUpdateSilent", "assignmentsOnUnDock", "selfAssignments", "assignmentScripts"];
        this.assigmentsOrder = ["onDock", "onUnDock", "onUpdate"];
        this.subComponentsOrder = ["internalId", "componentId", "isMain", "active", "numberInPartList", "assignments"];
        this.supersedingOrder = ["type", "parameter"];
        this.possibleChildOrder = ["group", "itemId", "componentId", "condition"];
        this.dimensioningOrder = ["type", "level", "from", "to", "visible", "labels"];
        this.rangeOrder = ["valueFrom", "valueTo", "step"];
        this.valueObjectOrder = ["value", "size", "labels", "condition", "thumbnail"];
    }
    normalizeComponent(obj) {
        var nc = {};
        nc = this.normalize(obj, this.componentOrder);
        if (nc.labels) {
            nc.labels = this.normalize(nc.labels, this.labelsOrder);
        }
        if (obj.parameters) {
            nc.parameters = [];
            obj.parameters.forEach((p) => {
                var np = this.normalize(p, this.parametersOrder);
                if (p.labels) {
                    np.labels = this.normalize(p.labels, this.labelsOrder);
                }
                if (p.validRange) {
                    np.validRange = this.normalize(p.validRange, this.rangeOrder);
                }
                if (p.valueObjects) {
                    np.valueObjects = [];
                    p.valueObjects.forEach((vo) => {
                        var nvo = this.normalizeValueObject(vo);
                        np.valueObjects.push(nvo);
                    });
                }
                if (p.conditionalGroups) {
                    np.conditionalGroups = [];
                    p.conditionalGroups.forEach((cg) => {
                        var ncg = this.normalizeValueObject(cg);
                        np.conditionalGroups.push(ncg);
                    });
                }
                if (!this.isEmptyObject(np))
                    nc.parameters.push(np);
            });
            if (nc.parameters.length === 0) {
                delete nc.parameters;
            }
        }
        if (obj.subComponents) {
            nc.subComponents = [];
            obj.subComponents.forEach((sc) => {
                var nsc = this.normalize(sc, this.subComponentsOrder);
                if (sc.assignments) {
                    nsc.assignments = this.normalize(sc.assignments, []);
                }
                if (this.isEmptyObject(nsc.assignments)) {
                    delete nsc.assignments;
                }
                if (sc.supersedings) {
                    nsc.supersedings = [];
                    sc.supersedings.forEach((sp) => {
                        var nsp = this.normalize(sp, this.supersedingOrder);
                        if (!this.isEmptyObject(nsp))
                            nsc.supersedings.push(nsp);
                    });
                    if (nsc.supersedings.length === 0) {
                        delete nsc.supersedings;
                    }
                }
                if (!this.isEmptyObject(nsc))
                    nc.subComponents.push(nsc);
            });
            if (nc.subComponents.length === 0) {
                delete nc.subComponents;
            }
        }
        if (obj.siblings) {
            nc.siblings = [];
            obj.siblings.forEach((sibling) => {
                var ns = this.normalizeConnection(sibling);
                if (!this.isEmptyObject(ns))
                    nc.siblings.push(ns);
            });
            if (nc.siblings.length === 0) {
                delete nc.siblings;
            }
        }
        if (obj.parentDockings) {
            nc.parentDockings = this.normalize(obj.parentDockings, this.dockingsOrder);
            if (obj.parentDockings.points) {
                nc.parentDockings.points = [];
                obj.parentDockings.points.forEach((point) => {
                    var npoint = this.normalizeConnection(point);
                    if (!this.isEmptyObject(npoint))
                        nc.parentDockings.points.push(npoint);
                });
                if (nc.parentDockings.points.length === 0) {
                    delete nc.parentDockings.points;
                }
            }
            if (obj.parentDockings.ranges) {
                nc.parentDockings.ranges = [];
                obj.parentDockings.ranges.forEach((range) => {
                    var nrange = this.normalizeConnection(range);
                    if (!this.isEmptyObject(nrange))
                        nc.parentDockings.ranges.push(nrange);
                });
                if (nc.parentDockings.ranges.length === 0) {
                    delete nc.parentDockings.ranges;
                }
            }
            if (obj.parentDockings.lines) {
                nc.parentDockings.lines = [];
                obj.parentDockings.lines.forEach((line) => {
                    var nline = this.normalizeConnection(line);
                    if (!this.isEmptyObject(nline))
                        nc.parentDockings.lines.push(nline);
                });
                if (nc.parentDockings.lines.length === 0) {
                    delete nc.parentDockings.lines;
                }
            }
            if (obj.parentDockings.lineRanges) {
                nc.parentDockings.lineRanges = [];
                obj.parentDockings.lineRanges.forEach((lineRange) => {
                    var nlineRange = this.normalizeConnection(lineRange);
                    if (!this.isEmptyObject(nlineRange))
                        nc.parentDockings.lineRanges.push(nlineRange);
                });
                if (nc.parentDockings.lineRanges.length === 0) {
                    delete nc.parentDockings.lineRanges;
                }
            }
            if (this.isEmptyObject(nc.parentDockings)) {
                delete nc.parentDockings;
            }
        }
        if (obj.childDockings) {
            nc.childDockings = this.normalize(obj.childDockings, this.dockingsOrder);
            if (obj.childDockings.points) {
                nc.childDockings.points = [];
                obj.childDockings.points.forEach((point) => {
                    var npoint = this.normalizeConnection(point);
                    if (!this.isEmptyObject(npoint))
                        nc.childDockings.points.push(npoint);
                });
                if (nc.childDockings.points.length === 0) {
                    delete nc.childDockings.points;
                }
            }
            if (this.isEmptyObject(nc.childDockings)) {
                delete nc.childDockings;
            }
        }
        if (obj.addOnSpots) {
            nc.addOnSpots = [];
            obj.addOnSpots.forEach((addOnSpot) => {
                var n_addOnSpot = this.normalize(addOnSpot, this.addOnSpotsOrder);
                if (!this.isEmptyObject(n_addOnSpot))
                    nc.addOnSpots.push(n_addOnSpot);
            });
            if (nc.addOnSpots.length === 0) {
                delete nc.addOnSpots;
            }
        }
        if (obj.possibleChildren) {
            nc.possibleChildren = [];
            obj.possibleChildren.forEach((possibleChildren) => {
                var n_possibleChildren = this.normalize(possibleChildren, this.possibleChildOrder);
                if (!this.isEmptyObject(n_possibleChildren))
                    nc.possibleChildren.push(n_possibleChildren);
            });
            if (nc.possibleChildren.length === 0) {
                delete nc.possibleChildren;
            }
        }
        if (obj.dimensionings) {
            nc.dimensionings = [];
            obj.dimensionings.forEach((dimension) => {
                var n_dimension = this.normalize(dimension, this.dimensioningOrder);
                if (!this.isEmptyObject(n_dimension))
                    nc.dimensionings.push(n_dimension);
            });
            if (nc.dimensionings.length === 0) {
                delete nc.dimensionings;
            }
        }
        return nc;
    }
    normalizeConnection(connectionObj) {
        var nconnection = this.normalize(connectionObj, this.connectionOrder);
        if (connectionObj.assignmentsOnDock) {
            nconnection.assignmentsOnDock = this.normalize(connectionObj.assignmentsOnDock, []);
        }
        if (this.isEmptyObject(nconnection.assignmentsOnDock)) {
            delete nconnection.assignmentsOnDock;
        }
        if (connectionObj.assignmentsOnUpdate) {
            nconnection.assignmentsOnUpdate = this.normalize(connectionObj.assignmentsOnUpdate, []);
        }
        if (this.isEmptyObject(nconnection.assignmentsOnUpdate)) {
            delete nconnection.assignmentsOnUpdate;
        }
        if (connectionObj.assignmentsOnUpdateSilent) {
            nconnection.assignmentsOnUpdateSilent = this.normalize(connectionObj.assignmentsOnUpdateSilent, []);
        }
        if (this.isEmptyObject(nconnection.assignmentsOnUpdateSilent)) {
            delete nconnection.assignmentsOnUpdateSilent;
        }
        if (connectionObj.assignmentsOnUnDock) {
            nconnection.assignmentsOnUnDock = this.normalize(connectionObj.assignmentsOnUnDock, []);
        }
        if (this.isEmptyObject(nconnection.assignmentsOnUnDock)) {
            delete nconnection.assignmentsOnUnDock;
        }
        if (connectionObj.selfAssignments) {
            nconnection.selfAssignments = this.normalize(connectionObj.selfAssignments, this.assigmentsOrder);
            if (connectionObj.selfAssignments.onDock) {
                nconnection.selfAssignments.onDock = this.normalize(connectionObj.selfAssignments.onDock, []);
            }
            if (this.isEmptyObject(nconnection.selfAssignments.onDock)) {
                delete nconnection.selfAssignments.onDock;
            }
            if (connectionObj.selfAssignments.onUpdate) {
                nconnection.selfAssignments.onUpdate = this.normalize(connectionObj.selfAssignments.onUpdate, []);
            }
            if (this.isEmptyObject(nconnection.selfAssignments.onUpdate)) {
                delete nconnection.selfAssignments.onUpdate;
            }
            if (connectionObj.selfAssignments.onUnDock) {
                nconnection.selfAssignments.onUnDock = this.normalize(connectionObj.selfAssignments.onUnDock, []);
            }
            if (this.isEmptyObject(nconnection.selfAssignments.onUnDock)) {
                delete nconnection.selfAssignments.onUnDock;
            }
        }
        if (connectionObj.assigmnentScripts) {
            nconnection.assigmnentScripts = this.normalize(connectionObj.assigmnentScripts, this.assigmentsOrder);
        }
        return nconnection;
    }
    normalizeValueObject(valueObject) {
        var nvo = this.normalize(valueObject, this.valueObjectOrder);
        if (nvo.labels) {
            nvo.labels = this.normalize(nvo.labels, this.labelsOrder);
        }
        return nvo;
    }
    normalize(obj, order) {
        var normalizedObj = {};
        // get all keys from object
        var allKeys = [];
        for (var key in obj) {
            allKeys.push(key);
        }
        // sort them in the order that the arrays above define
        allKeys.sort((a, b) => this.sortKeysByArray(a, b, order));
        // make copy of the object
        for (var i = 0; i < allKeys.length; i++) {
            let attribute = allKeys[i];
            if (attribute in obj) {
                if ((Array.isArray(obj[attribute]) && obj[attribute].length === 0) || ((typeof (obj[attribute]) === 'object' || typeof (obj[attribute]) === 'function') && (Object.keys(obj[attribute]).length === 0)) || obj[attribute] === "") {
                    /* skip */
                }
                else {
                    var take = obj[attribute];
                    normalizedObj[attribute] = take;
                }
            }
            else {
                this.outputChannel?.appendLine("WARN: attribute " + attribute + " not copied during normalize. Report this as bug.");
            }
        }
        return normalizedObj;
    }
    isEmptyObject(obj) {
        return ((typeof (obj) === 'object' || typeof (obj) === 'function') && (Object.keys(obj).length === 0));
    }
    sortKeysByArray(a, b, order) {
        var _a = order.indexOf(a);
        var _b = order.indexOf(b);
        if (_a + _b === -2) {
            if (a < b) {
                return -1;
            }
            else if (a === b) {
                return 0;
            }
            else {
                return 1;
            }
        }
        else {
            if (_a < 0)
                _a = 999;
            if (_b < 0)
                _b = 999;
            return _a - _b;
        }
    }
}
exports.Normalizer = Normalizer;
//# sourceMappingURL=normalizer.js.map